import polars as pl
import sqlglot
from jinja2 import Environment

from focus_converter.configs.base_config import (
    ConversionPlan,
    SQLConditionConversionArgs,
)
from focus_converter.conversion_functions.validations import ColumnValidator

# used in sql queries
# TODO: Make it configurable according to context if needed
DEFAULT_SQL_TABLE_NAME = "cost_data"


SQL_TEMPLATE_CONDITION_PLAN = """
select *, case {{ CASE_CONDITIONS }} ELSE {{ DEFAULT_VALUE }} END as {{ NEW_COLUMN }} from {{ TABLE_NAME }}
"""


class SQLFunctions:
    @staticmethod
    def create_sql_context(lf: pl.LazyFrame):
        sql_context = pl.SQLContext()
        sql_context.register(DEFAULT_SQL_TABLE_NAME, lf)
        return sql_context

    @staticmethod
    def __validate_sql_query__(sql_query):
        try:
            sqlglot.transpile(sql_query)
        except sqlglot.errors.ParseError:
            raise ValueError(f"Invalid sql produced: {sql_query}")

    @classmethod
    def eval_sql_conditions(
        cls, plan: ConversionPlan, column_alias, column_validator: ColumnValidator
    ):
        conversion_args = SQLConditionConversionArgs.model_validate(
            plan.conversion_args
        )

        case_statements = []
        for condition in conversion_args.conditions:
            case_statements.append(condition)

        template = Environment().from_string(SQL_TEMPLATE_CONDITION_PLAN)
        sql_query = template.render(
            CASE_CONDITIONS=" ".join(case_statements),
            DEFAULT_VALUE=conversion_args.default_value,
            NEW_COLUMN=column_alias,
            TABLE_NAME=DEFAULT_SQL_TABLE_NAME,
        )

        # validate sql query structure
        cls.__validate_sql_query__(sql_query=sql_query)

        # destructure sql query and validate column names
        column_validator.map_sql_query(sql_query=sql_query, plan=plan)

        return sql_query

    @classmethod
    def eval_sql_query(
        cls, plan: ConversionPlan, column_validator: ColumnValidator, **_kwargs
    ):
        template = Environment().from_string(plan.conversion_args)
        sql_query = template.render(
            TABLE_NAME=DEFAULT_SQL_TABLE_NAME,
        )

        # validate sql query structure
        cls.__validate_sql_query__(sql_query=sql_query)

        # destructure sql query and validate column names
        column_validator.map_sql_query(sql_query=sql_query, plan=plan)

        return sql_query



import logging

import pandas as pd
import polars as pl
import pyarrow
import pyarrow.dataset as ds

from focus_converter.common.cli_options import (
    DATA_FORMAT_OPTION,
    PARQUET_DATA_FORMAT_OPTION,
)
from focus_converter.conversion_functions import STATIC_CONVERSION_TYPES
from focus_converter.converter import FocusConverter


class ProviderSensor:
    """
    Senses provider automatically using first few rows of an input file.
    Can also distinguish between various file formats.
    """

    provider: str = None
    data_format: DATA_FORMAT_OPTION = None
    parquet_data_format: PARQUET_DATA_FORMAT_OPTION = None

    def __init__(self, base_path):
        self.__base_path__ = base_path

    def __try_load_csv_file__(self):
        df = pd.read_csv(self.__base_path__)
        return df.head(10)

    def __try_load_parquet_fragments__(self):
        dataset = ds.dataset(self.__base_path__)
        return pl.scan_pyarrow_dataset(dataset).head(10).collect().to_pandas()

    def __try_load_parquet__(self):
        return pl.scan_parquet(self.__base_path__).head(10).collect().to_pandas()

    def __sense_file_format__(self):
        try:
            data_sample = self.__try_load_parquet_fragments__()
            self.data_format = DATA_FORMAT_OPTION.PARQUET
            self.parquet_data_format = PARQUET_DATA_FORMAT_OPTION.DATASET
            return data_sample
        except pyarrow.lib.ArrowInvalid as e:
            logging.debug(f"Not parquet dataset, {str(e)}")

        try:
            data_sample = self.__try_load_parquet__()
            self.data_format = DATA_FORMAT_OPTION.PARQUET
            self.parquet_data_format = PARQUET_DATA_FORMAT_OPTION.FILE
            return data_sample
        except pl.exceptions.ComputeError as e:
            logging.debug(f"Not parquet file, {str(e)}")

        try:
            data_sample = self.__try_load_csv_file__()
            self.data_format = DATA_FORMAT_OPTION.CSV
            return data_sample
        except Exception as e:
            logging.debug(f"Not csv file, {str(e)}")

        raise RuntimeError(
            f"Exhausted all read methods, no suitable file format found for input: {self.__base_path__}"
        )

    def __sense_provider__(self, data_sample):
        computed_provider_name = None
        matched_column_names_count = 0

        converter = FocusConverter()
        converter.load_provider_conversion_configs()
        for provider_name in converter.plans.keys():
            for plan in converter.plans[provider_name]:
                if plan.conversion_type == STATIC_CONVERSION_TYPES.SET_COLUMN_DTYPES:
                    plan_column_names = [
                        column_obj["column_name"]
                        for column_obj in plan.conversion_args["dtype_args"]
                    ]
                    matched_columns = set(plan_column_names) & set(data_sample.columns)
                    if len(matched_columns) > matched_column_names_count:
                        computed_provider_name = provider_name
                        matched_column_names_count = len(matched_columns)

        if computed_provider_name is None:
            raise RuntimeError(
                "Failed to find a suitable provider for the given dataset, either provider is not configured yet or data might be corrupted."
            )

        self.provider = computed_provider_name

    def load(self):
        data_sample = self.__sense_file_format__()
        self.__sense_provider__(data_sample=data_sample)



import polars as pl

from focus_converter.configs.base_config import ConversionPlan
from focus_converter.configs.string_transform_args import (
    StringSplitArgument,
    StringTransformArgs,
)
from focus_converter.conversion_functions.validations import ColumnValidator


class StringFunctions:
    @staticmethod
    def convert(plan: ConversionPlan, column_alias, column_validator: ColumnValidator):
        conversion_args = StringTransformArgs.model_validate(plan.conversion_args)

        # add to column validator and check if source column exists
        column_validator.map_non_sql_plan(plan=plan, column_alias=column_alias)

        column_expr = pl.col(plan.column)

        for step in conversion_args.steps:
            if step == "lower":
                column_expr = column_expr.str.to_lowercase()
            elif step == "upper":
                column_expr = column_expr.str.to_uppercase()
            elif step == "title":
                column_expr = column_expr.str.to_titlecase()
            elif isinstance(step, StringSplitArgument):
                column_expr = column_expr.str.split(step.split_by)
                if step.index:
                    column_expr = column_expr.list.get(index=step.index)
            else:
                raise ValueError(f"Invalid step {step} in conversion plan {plan}")

        return column_expr.alias(column_alias)



import os
import tempfile
from typing import Annotated

import polars as pl
import pyarrow.dataset as ds
import typer

from focus_converter.converter import FocusConverter
from focus_converter.data_loaders.data_loader import DataFormats

app = typer.Typer(name="FOCUS export converted sample data", add_completion=False)


class GenerateSampleConvertedFOCUSDataset:
    @staticmethod
    def __helper__(output_dir, test_dataset_path, provider):
        with tempfile.TemporaryDirectory() as temp_dir:
            converter = FocusConverter()
            converter.load_provider_conversion_configs()
            converter.load_data(
                data_path=test_dataset_path,
                data_format=DataFormats.CSV,
            )
            converter.configure_data_export(
                export_path=os.path.join(temp_dir, "data"),
                export_include_source_columns=False,
            )
            converter.prepare_horizontal_conversion_plan(provider=provider)
            converter.convert()

            csv_output_path = os.path.join(output_dir, f"{provider}.csv")
            dataset = ds.dataset(temp_dir, format="parquet")
            df = pl.scan_pyarrow_dataset(dataset).collect()
            df.write_csv(csv_output_path)

            written_df = pl.scan_csv(csv_output_path).collect()

            # ensure that written dataframe has column and rows, greater than 0
            assert written_df.shape[0] > 0
            assert written_df.shape[1] > 0

    @classmethod
    def generate_aws_sample_dataset(cls, output_dir):
        cls.__helper__(
            output_dir=output_dir,
            test_dataset_path="tests/provider_config_tests/aws/sample-anonymous-aws-export-dataset.csv",
            provider="aws-cur",
        )

    @classmethod
    def generate_oci_sample_dataset(cls, output_dir):
        cls.__helper__(
            output_dir=output_dir,
            test_dataset_path="tests/provider_config_tests/oci/reports_cost-csv_0000000030000269.csv",
            provider="oci",
        )

    @classmethod
    def generate_azure_sample_dataset(cls, output_dir):
        cls.__helper__(
            output_dir=output_dir,
            test_dataset_path="tests/provider_config_tests/azure/sample-anonymous-ea-export-dataset.csv",
            provider="azure",
        )


@app.command("export-focus-dataset", help="Converts source cost data to FOCUS format")
def export_converted_sample_data(
    output_dir: Annotated[
        str,
        typer.Option(help="Write provider csvs to this path."),
    ]
):
    GenerateSampleConvertedFOCUSDataset.generate_aws_sample_dataset(
        output_dir=output_dir
    )
    GenerateSampleConvertedFOCUSDataset.generate_oci_sample_dataset(
        output_dir=output_dir
    )
    GenerateSampleConvertedFOCUSDataset.generate_azure_sample_dataset(
        output_dir=output_dir
    )


if __name__ == "__main__":
    app()



import polars as pl

from focus_converter.configs.base_config import ConversionPlan, LookupConversionArgs
from focus_converter.conversion_functions.validations import ColumnValidator


class LookupFunction:
    @classmethod
    def map_values_using_lookup(
        cls, plan: ConversionPlan, column_alias, column_validator: ColumnValidator
    ):
        conversion_args = LookupConversionArgs.model_validate(plan.conversion_args)
        reference_data_lf = (
            pl.scan_csv(conversion_args.reference_dataset_path)
            .select([conversion_args.source_value, conversion_args.destination_value])
            .with_columns(
                [
                    pl.col(conversion_args.source_value).cast(pl.Utf8),
                ]
            )
            .rename({conversion_args.destination_value: column_alias})
        )

        # add to column validator and check if source column exists
        column_validator.map_non_sql_plan(plan=plan, column_alias=column_alias)

        return {
            "other": reference_data_lf,
            "left_on": plan.column,
            "how": "left",
            "right_on": conversion_args.source_value,
        }



from typing import List, Literal, Optional, Union

from pydantic import BaseModel


class StringSplitArgument(BaseModel):
    operation_type: Literal["split"]
    split_by: str
    index: Optional[int] = None


class StringTransformArgs(BaseModel):
    steps: List[
        Union[
            StringSplitArgument,
            Literal[
                "lower",
                "upper",
                "title",
            ],
        ]
    ]
